;(function () {
  if(window.NativeBridge){
    return;
  }

  var BRIDGE_SCHEME = 'jsbridge';
  var QUEUE_HAS_MESSAGE = 'has_message/';
  var FETCH_QUEUE = 'return_data/';

  //消息
  function Message() {
    this.moduleName;  //string
    this.methodName;  //string
    this.data;  //json 数组
    this.jsCallbackId; //需回调的js callback 函数id
    this.javaCallbackId;  //回调的java callback 函数id 暂不支持
  }

  var messageIFrame;
  var sendMessageQueue = [];
  var jsCallbacks = [];
  var uniqueId = 1;

  function _handleMessageFromNative(messageJSON) {
    setTimeout(function () {
      var message = JSON.parse(messageJSON);
      //回调js
      if (message.jsCallbackId) {
        var jsCallback = jsCallbacks[message.jsCallbackId]
        if (jsCallback) {
          jsCallback.apply(null, message.data)
          delete jsCallback[message.jsCallbackId];
        }
      }
      //直接调用js
      else {
        //需要回传
        if (message.javaCallbackId) {
          var callbackMessage = new Message();
          callbackMessage.javaCallbackId = message.javaCallbackId;
        }

        var jsModule;
        if (!message.moduleName || message.moduleName == "window") {
          jsModule = window;
        } else {
          jsModule = window[message.moduleName]
        }

        if (jsModule && jsModule[message.methodName]) {
          var args = message.data;
          jsModule[message.methodName].apply(jsModule, args);
        } else {
          try{
            console.error("Brigde: " + (jsModule || message.moduleName) + "." + message.methodName + " not fund");
          } catch(e){}
        }
      }
    }, 0);
  }

  //sendMessage add message, 触发native处理 sendMessage
  function sendMessageToNative(message) {
    sendMessageQueue.push(message);
    getMessageIFrame(document).src = BRIDGE_SCHEME + "://" + QUEUE_HAS_MESSAGE;
  }

  function _fetchQueue() {
    var messageQueueString = JSON.stringify(sendMessageQueue);
    sendMessageQueue = [];
    getMessageIFrame(document).src =
        BRIDGE_SCHEME + "://" + FETCH_QUEUE + encodeURIComponent(messageQueueString);
  }

  /**
   *
   * @param moduleName
   * @param methodName
   * @param callback 回调函数
   * @param args 对象数组
   */
  function callNativeModule(moduleName, methodName, args, callback) {
    var message = new Message();
    message.moduleName = moduleName;
    message.methodName = methodName;

    if (callback) {
      var callbackId = 'cb_' + (uniqueId++) + '_' + new Date().getTime();
      jsCallbacks[callbackId] = callback;
      message.jsCallbackId = callbackId;
    }
    message.data = args;
    sendMessageToNative(message);
  }

  /**
   *  NativeModule方法本地化，使其可以直接调用 ，为了兼容javaInterface 模式
   */
  function localizeNativeModule(content) {
    var moduleInfoArray = JSON.parse(content);
    var i = moduleInfoArray.length
    while (i--) {
      var moduleInfo = moduleInfoArray[i];
      var moduleName = moduleInfo.moduleName;
      var methodInfos = moduleInfo.moduleMethods;
      var j = methodInfos.length
      while (j--) {
        var methodInfo = methodInfos[j];
        var methodName = methodInfo.methodName;

        var adpteNativeModule;
        if (moduleName && moduleName != "window") {
          if (!window[moduleName]) {
            window[moduleName] = {};
          }
          adpteNativeModule = window[moduleName];
        } else {
          adpteNativeModule = window;
        }

        var args = [moduleName, methodName];
        adpteNativeModule[methodName] = Function.apply(null,
           [" var args = Array.prototype.slice.call(arguments);"
           + "var callback;"
           + "if(args.length > 0 && typeof args[args.length -1] == 'function'){"
           + "  callback = args[args.length -1];"
           + "  args =  args.slice(0, args.length -1);"
           + "}"
           + "NativeBridge.invoke('" + moduleName + "','" + methodName + "'," + "args, callback);"]);
      }
    }
  }

  /**
   *创建iframe 对象
   * @param doc document
   */
  function getMessageIFrame(doc) {
    if(! messageIFrame ) {
      var iframe = doc.createElement('iframe');
      iframe.style.display = 'none';
      messageIFrame = doc.documentElement.appendChild(iframe);
    }
    return messageIFrame;
  }

  var NativeBridge = window.NativeBridge = {
    invoke: callNativeModule,
    fetchQueue: _fetchQueue,
    handleMessageFromNative: _handleMessageFromNative
  };

  var readyEvent = document.createEvent('Events');
  readyEvent.initEvent('NativeBridgeReady');
  readyEvent.bridge = NativeBridge;
  document.dispatchEvent(readyEvent);

  console.log("NativeBridge Ready")
  localizeNativeModule("${{adapte_info}}");
})();
